@@ -30,7 +30,7 @@
 ;;; Code:
 (provide 'navi2ch-multibbs)
 (defconst navi2ch-multibbs-ident
-  "$Id: navi2ch-multibbs.el,v 1.41 2006/06/10 13:53:52 knok Exp $")
+  "$Id: navi2ch-multibbs.el,v 1.45.2.4 2007/05/02 01:48:08 nanashi Exp $")
 
 (eval-when-compile (require 'cl))
 (require 'navi2ch-http-date)
@@ -52,6 +52,7 @@ FUNC-ALIST $B$O0J2<$NDL$j(B
  (url-to-board		. URL-TO-BOARD-FUNC)
  (url-to-article	. URL-TO-ARTICLE-FUNC)
  (send-message		. SEND-MESSAGE-FUNC)
+ (extract-post          . EXTRACT-POST-FUNC)
  (send-success-p	. SEND-MESSAGE-SUCCESS-P-FUNC)
  (error-string		. ERROR-STRING-FUNC)
  (board-update		. BOARD-UPDATE-FUNC)
@@ -79,9 +80,19 @@ URL-TO-ARTICLE-FUNC(URL):
 URL $B$+$i(B article $B$KJQ49$9$k!#(B
 
 SEND-MESSAGE-FUNC(FROM MAIL MESSAGE
-		  SUBJECT BBS KEY TIME BOARD ARTICLE):
+		  SUBJECT BBS KEY TIME BOARD ARTICLE POST):
     MESSAGE $B$rAw?.$9$k!#(B
 
+EXTRACT-POST-FUNC(OLD-POST BUFFER):
+    MESSAGE $B$N:FAw?.$K;H$&>pJs$r<h$j=P$9!#<h$j=P$7$?%G!<%?$r(B
+    $BJV$jCM$H$7$FJV$9$H!":FAw$N$?$a$K(BSEND-MESSAGE-FUNC$B$r8F$S=P$9(B
+    $B$H$-$K!"$=$NCM$,(BPOST$B0z?t$KB+G{$5$l$^$9!#(B
+     
+    BUFFER$B$K$O!"(BSEND-MESSAGE-FUNC$B$,JV$7$?(BPROC$B$+$i<h$j=P$7$?%3%s(B
+    $B%F%s%D$r%G%3!<%I$7$?$b$N$,A^F~$5$l$F$$$^$9!#:FAw$r7+$jJV$9(B
+    $B>l9g!":G8e$K(BEXTRACT-POST-FUNC$B$,JV$7$?CM$,(BOLD-POST$B$KB+G{$5$l(B
+    $B$F$$$^$9!#(B
+
 SEND-MESSAGE-SUCCESS-P-FUNC(PROC):
     PROC $B$NAw?.%;%C%7%g%s$,@.8y$7$F$$$l$P(B non-nil $B$r!"(B
     $B<:GT$7$?$i(B nil $B$r!":F;n9T2DG=$J<:GT$J$i(B 'retry $B$rJV$9!#(B
@@ -142,8 +153,8 @@ SPEC $B$O(B (BBSTYPE [ARG]...)$B!#(B
 $B<B:]$K$O!"(Bcallback $B$rDj5A$9$k$N$KI,MW$J(B BBSTYPE $B$JHD$N(B coding-system
 $B$K$h$k(B decode, encode $B=hM}$r(B BODY $B$rI>2A$9$kA08e$K9T$J$&!"(BNAME $B$H$$$&(B
 $B0z?t(B [ARG]... $B$r;}$D4X?t$,Dj5A$5$l$k!#(B"
-  (let ((bbstype (gensym "--bbstype--"))
-	(decoding (gensym "--decoding--"))
+  (let ((bbstype (make-symbol "--bbstype--"))
+	(decoding (make-symbol "--decoding--"))
 	docstring)
     (when (stringp (car body))
       (setq docstring (car body))
@@ -180,7 +191,7 @@ SPEC $B$O(B (BBSTYPE [ARG]...)$B!#(B
 	(cons (cons bbstype variable-alist)
 	      navi2ch-multibbs-variable-alist)))
 
-(defsubst navi2ch-multibbs-get-func-from-board
+(defun navi2ch-multibbs-get-func-from-board
   (board func &optional default-func)
   (navi2ch-multibbs-get-func
    (navi2ch-multibbs-get-bbstype board)
@@ -237,20 +248,6 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 	      h (- h 0)))
       (cons h l))))
 
-(defvar navi2ch-multibbs-send-message-retry-confirm-function #'yes-or-no-p)
-
-(defun navi2ch-multibbs-send-message-retry-confirm (board)
-  (let ((func (or (navi2ch-fboundp
-		   navi2ch-multibbs-send-message-retry-confirm-function)
-		  #'yes-or-no-p))
-	spid)
-    (unwind-protect
-	(let ((result (funcall func "Retry? ")))
-	  (when result
-	    (setq spid (navi2ch-board-load-spid board)))
-	  result)
-      (navi2ch-board-save-spid board spid))))
-
 (defun navi2ch-multibbs-send-message-error-string (board proc)
   (let* ((func (navi2ch-multibbs-get-func
 		(navi2ch-multibbs-get-bbstype board)
@@ -267,6 +264,8 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
   (let* ((bbstype      (navi2ch-multibbs-get-bbstype board))
 	 (send         (navi2ch-multibbs-get-func
 			bbstype 'send-message 'navi2ch-2ch-send-message))
+	 (extract-post (navi2ch-multibbs-get-func
+			bbstype 'extract-post 'navi2ch-2ch-extract-post))
 	 (success-p    (navi2ch-multibbs-get-func
 			bbstype 'send-success-p
 			'navi2ch-2ch-send-message-success-p))
@@ -287,10 +286,11 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 					    navi2ch-net-http-proxy-password))
 	 (tries 2)			; $BAw?.;n9T$N:GBg2s?t(B
 	 (message-str "send message...")
-	 (result 'retry))
+	 (result 'retry)
+	 (post-data nil))
     (dotimes (i tries)
       (let ((proc (funcall send from mail message subject bbs key time
-			   board article)))
+			   board article post-data)))
 	(message message-str)
 	(setq result (funcall success-p proc))
 	(cond ((eq result 'retry)
@@ -299,13 +299,14 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 		   (insert (decode-coding-string
 			    (navi2ch-net-get-content proc)
 			    (navi2ch-board-get-coding-system board)))
+		   (setq post-data (funcall extract-post post-data (current-buffer)))
 		   (navi2ch-replace-html-tag-with-buffer)
 		   (goto-char (point-min))
 		   (while (re-search-forward "[ \t]*\n\\([ \t]*\n\\)*" nil t)
 		     (replace-match "\n"))
 		   (delete-other-windows)
 		   (switch-to-buffer (current-buffer))
-		   (unless (navi2ch-multibbs-send-message-retry-confirm board)
+		   (unless (y-or-n-p "Retry? ")
 		     (return nil))))
 	       (sit-for navi2ch-message-retry-wait-time)
 	       (setq message-str "re-send message..."))
@@ -342,19 +343,11 @@ START $B$,(B non-nil $B$J$i$P%l%9HV9f(B START $B$+$i$N:9J,$r<hF@$9$k!#(B
   (let ((file (navi2ch-article-get-file-name board article))
 	(time (cdr (assq 'time article)))
 	url header kako-p)
-    (if (and (navi2ch-enable-readcgi-p
-	      (navi2ch-board-get-host board)))
-	(progn
-	  (setq url (navi2ch-article-get-readcgi-raw-url board article start))
-	  (setq header (navi2ch-net-update-file-with-readcgi url file
-							     time start))
-	  ;; read.cgi $B$r;H$C$F$F%(%i!<$K$J$k$N$O%[%s%H$K%(%i!<$N$H$-(B
-	  (setq kako-p (navi2ch-net-get-state 'kako header)))
-      (setq url (navi2ch-article-get-url board article))
-      (setq header (if start
-		       (navi2ch-net-update-file-diff url file time)
-		     (navi2ch-net-update-file url file time)))
-      (setq kako-p (navi2ch-net-get-state 'error header)))
+    (setq url (navi2ch-article-get-url board article))
+    (setq header (if start
+		     (navi2ch-net-update-file-diff url file time)
+		   (navi2ch-net-update-file url file time)))
+    (setq kako-p (navi2ch-net-get-state 'error header))
     (when kako-p
       (setq url (navi2ch-article-get-kako-url board article))
       (setq header (navi2ch-net-update-file url file))
@@ -420,57 +413,57 @@ START $B$,(B non-nil $B$J$i$P%l%9HV9f(B START $B$+$i$N:9J,$r<hF@$9$k!#(B
 (defvar navi2ch-2ch-send-message-last-board nil)
 
 (defun navi2ch-2ch-send-message
-  (from mail message subject bbs key time board article)
-  (let ((url         (navi2ch-board-get-bbscgi-url board))
-	(referer     (navi2ch-board-get-uri board))
-	(spid        (navi2ch-board-load-spid board))
-	(param-alist (list
-		      (cons "submit" "$B=q$-9~$`(B")
-		      (cons "FROM"   (or from ""))
-		      (cons "mail"   (or mail ""))
-		      (cons "bbs"    bbs)
-		      (cons "time"   time)
-		      (cons "MESSAGE" message)
-		      (if subject
-			  (cons "subject" subject)
-			(cons "key"    key)))))
+  (from mail message subject bbs key time board article &optional post)
+  (let* ((url         (navi2ch-board-get-bbscgi-url board))
+	 (referer     (navi2ch-board-get-uri board))
+	 (param-alist (list
+		       (cons "submit" "$B=q$-9~$`(B")
+		       (cons "FROM"   (or from ""))
+		       (cons "mail"   (or mail ""))
+		       (cons "bbs"    bbs)
+		       (cons "time"   time)
+		       (cons "MESSAGE" message)
+		       (if subject
+			   (cons "subject" subject)
+			 (cons "key"    key))))
+	 (coding-system (navi2ch-board-get-coding-system board))
+	 (cookies (navi2ch-net-match-cookies url)))
+    (dolist (param post)
+      (unless (assoc (car param) param-alist)
+	(push param param-alist)))
     (setq navi2ch-2ch-send-message-last-board board)
-    (setq spid
-	  (when (and (consp spid)
-		     (navi2ch-compare-times (cdr spid) (current-time)))
-	    (car spid)))
     (let ((proc
 	   (navi2ch-net-send-request
 	    url "POST"
 	    (list (cons "Content-Type" "application/x-www-form-urlencoded")
-		  (cons "Cookie" (concat "NAME=" from "; MAIL=" mail
-					 (if spid (concat "; SPID=" spid
-							  "; PON=" spid))
-					 (if (navi2ch-be2ch-login-p)
-					     (concat "; MDMD=" navi2ch-be2ch-mdmd
-						     "; DMDM=" navi2ch-be2ch-dmdm))))
+		  (cons "Cookie"
+			(navi2ch-net-cookie-string cookies coding-system))
 		  (cons "Referer" referer))
 	    (navi2ch-net-get-param-string param-alist
-					  (navi2ch-board-get-coding-system board)))))
-      (setq spid (navi2ch-net-send-message-get-spid proc))
-      (if spid (navi2ch-board-save-spid board spid))
+					  coding-system))))
+      (navi2ch-net-update-cookies url proc coding-system)
+      (navi2ch-net-save-cookies)
       proc)))
 
 (defun navi2ch-2ch-article-to-url
   (board article &optional start end nofirst)
   "BOARD, ARTICLE $B$+$i(B url $B$KJQ49!#(B
 START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
-  (let ((url (navi2ch-board-get-readcgi-url board)))
-    (setq url (concat url (cdr (assq 'artid article)) "/"))
-    (if (numberp start)
-	(setq start (number-to-string start)))
-    (if (numberp end)
-	(setq end (number-to-string end)))
-    (if (equal start end)
-	(concat url start)
-      (concat url
-	      start (and (or start end) "-") end
-	      (and nofirst "n")))))
+  (let ((uri (navi2ch-board-get-uri board))
+	(start (if (numberp start)
+		   (number-to-string start)
+		 start))
+	(end (if (numberp end)
+		 (number-to-string end)
+	       end)))
+    (if (string-match "\\(.+\\)/\\([^/]+\\)/$" uri)
+	(format "%s/test/read.cgi/%s/%s/%s"
+		(match-string 1 uri) (match-string 2 uri)
+		(cdr (assq 'artid article))
+		(if (equal start end)
+		    (or start "")
+		  (concat start (and (or start end) "-") end
+			  (and nofirst "n")))))))
 
 (defun navi2ch-2ch-send-message-success-p (proc)
   (navi2ch-net-send-message-success-p
@@ -487,14 +480,11 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 (defun navi2ch-2ch-board-update (board)
   (let ((file (navi2ch-board-get-file-name board))
 	(time (cdr (assq 'time board))))
-    (if navi2ch-board-enable-readcgi
-	(navi2ch-net-update-file-with-readcgi
-	 (navi2ch-board-get-readcgi-raw-url board) file time)
-      (let ((url (navi2ch-board-get-url
-		  board (if navi2ch-board-use-subback-html
-			    navi2ch-board-subback-file-name)))
-	    (func (navi2ch-multibbs-subject-callback board)))
-	(navi2ch-net-update-file url file time func)))))
+    (let ((url (navi2ch-board-get-url
+		board (if navi2ch-board-use-subback-html
+			  navi2ch-board-subback-file-name)))
+	  (func (navi2ch-multibbs-subject-callback board)))
+      (navi2ch-net-update-file url file time func))))
 
 (defun navi2ch-2ch-board-get-file-name (board &optional file-name)
   (let ((uri (navi2ch-board-get-uri board)))
@@ -508,4 +498,27 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 				   navi2ch-board-subject-file-name)
 			       (match-string 1 uri)))))))
 
+(defun navi2ch-2ch-extract-post (old-post buffer)
+  ;; Get hana and mogera from following string.
+  ;; <input type=hidden name="hana" value="mogera">  
+  (with-current-buffer buffer
+    (save-excursion
+      (save-match-data
+	(goto-char (point-min))
+	(let ((case-fold-search t)
+	      (re "\\<%s=\\(\"\\([^\"]*\\)\"\\|[^\"> \r\n\t]*\\)")
+	      r)
+	  (while (re-search-forward "<input\\>[^>]+>" nil t)
+	    (let ((str (match-string 0)) name value)
+	      (and (string-match (format re "name") str)
+		   (setq name (or (match-string 2 str)
+				  (match-string 1 str)))
+		   (string-match (format re "value") str)
+		   (setq value (or (match-string 2 str)
+				   (match-string 1 str)))
+		   (setq name (navi2ch-replace-html-tag name)
+			 value (navi2ch-replace-html-tag value))
+		   (push (cons name value) r))))
+	  (nreverse r))))))
+
 ;;; navi2ch-multibbs.el ends here
